/*------------------------------------------------------------------------------
 * Copyright (c) 2010 Serge A. Zaitsev
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 * THE SOFTWARE.
 *----------------------------------------------------------------------------*/

#ifndef _JSMN_H_
#define _JSMN_H_

#include <stddef.h>

#ifdef __cplusplus
extern "C" {
#endif

#define JSMN_PARENT_LINKS

#define JSON_SERIALIZE_NAME                             "\"%s\":"
#define JSON_SERIALIZE_VALUE_INT                        "%d"
#define JSON_SERIALIZE_VALUE_STRING                     "\"%s\""
#define JSON_SERIALIZE_ITEM_OBJECT_START                "{"
#define JSON_SERIALIZE_ITEM_OBJECT_END                  "}"
#define JSON_SERIALIZE_ITEM_ARRAY_START                 "["
#define JSON_SERIALIZE_ITEM_ARRAY_END                   "]"
#define JSON_SERIALIZE_ITEM_SEPERATION                  ","
#define JSON_SERIALIZE_NEW_LINE                         "\n"

/**
 * JSON type identifier. Basic types are:
 *  o Object
 *  o Array
 *  o String
 *  o Other primitive: number, boolean (true/false) or null
 */
typedef enum {
    JSMN_UNDEFINED = 0,
    JSMN_OBJECT = 1,
    JSMN_ARRAY = 2,
    JSMN_STRING = 3,
    JSMN_PRIMITIVE = 4
} jsmntype_t;

enum jsmnerr {
    /* Not enough tokens were provided */
    JSMN_ERROR_NOMEM = -1,
    /* Invalid character inside JSON string */
    JSMN_ERROR_INVAL = -2,
    /* The string is not a full JSON packet, more bytes expected */
    JSMN_ERROR_PART = -3,
};

/**
 * JSON token description.
 * type     type (object, array, string etc.)
 * start    start position in JSON data string
 * end      end position in JSON data string
 */
typedef struct {
    jsmntype_t type;
    int start;
    int end;
    int size;
#ifdef JSMN_PARENT_LINKS
    int parent;
#endif
} jsmntok_t;

/**
 * JSON parser. Contains an array of token blocks available. Also stores
 * the string being parsed now and current position in that string
 */
typedef struct {
    unsigned int pos; /* offset in the JSON string */
    unsigned int toknext; /* next token to allocate */
    int toksuper; /* superior token node, e.g parent object or array */
} jsmn_parser;

/**
 * Parameters used in serialization
 */
typedef struct jsmn_serialize_params_s {
    char *psSerializedBuffer;
    unsigned int bufferMaxLen;

    int indentlevel;
    int indentPerLevel;
} jsmn_serialize_params_t;

/**
 * Create JSON parser over an array of tokens
 */
void jsmn_init(jsmn_parser *parser);

/**
 * Run JSON parser. It parses a JSON data string into and array of tokens, each describing
 * a single JSON object.
 */
int jsmn_parse(jsmn_parser *parser, const char *js, size_t len, jsmntok_t *tokens, unsigned int num_tokens);

/*
 * Read JSON from stdin and print its content to stdout.
 * The output looks like YAML, but I'm not sure if it's really compatible.
 */
int jsmn_dump(const char *js, jsmntok_t *t, size_t count, int indent);

/*
 * Compare whether string s is equal to tok's value
 */
int jsmn_eq(const char *json, jsmntok_t *tok, const char *s);

/**
 * Helper to adjust after string w/ length written is written to *pBuf
 */
int jsmn_puts(char **pBuf, unsigned int *pBufMaxLen, int *pWrittenTotal, int written);

/**
 * Helper to puts string in buf
 */
int jsmn_append_string(char *buf, unsigned int bufMaxLen, const char *string);

/**
 * Helper to snprintf in *pBuf
 */
int jsmn_snprintf(char **pBuf, unsigned int *pBufMaxLen, int *pWrittenTotal, char const* fmt, ...);

/**
 * Helper to puts indent " " in *pBuf
 */
int jsmn_append_indent(char **pBuf, unsigned int *pBufMaxLen, int *pWrittenTotal, int indentlevel, int indent);


#ifdef __cplusplus
}
#endif

#endif///_JSMN_H_
