/*------------------------------------------------------------------------------
 * Copyright (c) 2010 Serge A. Zaitsev
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 * THE SOFTWARE.
 *----------------------------------------------------------------------------*/

///////////////////////////////////////////////////////////////////////////////////////////////////
/// @file   jsmn.c
/// @brief  jsmn (pronounced like 'jasmine') is a minimalistic JSON parser in C
/// @author Serge A. Zaitsev
///////////////////////////////////////////////////////////////////////////////////////////////////

//-------------------------------------------------------------------------------------------------
//  Include Files
//-------------------------------------------------------------------------------------------------
#include <linux/string.h>
#include <stdarg.h>
#include <stdio.h>
#include <stdlib.h>

#include <bootlogo/jsmn.h>

//-------------------------------------------------------------------------------------------------
//  Local Compiler Options
//-------------------------------------------------------------------------------------------------

//-------------------------------------------------------------------------------------------------
//  Debug Defines
//-------------------------------------------------------------------------------------------------
#ifndef JSMN_DEBUG
#define JSMN_DEBUG                                      printf
#endif //#ifndef JSMN_DEBUG

//-------------------------------------------------------------------------------------------------
//  Local Defines
//-------------------------------------------------------------------------------------------------
#define JSON_MAX_SERIALIZE_LEN                          1024

//-------------------------------------------------------------------------------------------------
//  Local Structures
//-------------------------------------------------------------------------------------------------

//-------------------------------------------------------------------------------------------------
//  Global Variables
//-------------------------------------------------------------------------------------------------

//-------------------------------------------------------------------------------------------------
//  Local Variables
//-------------------------------------------------------------------------------------------------

//-------------------------------------------------------------------------------------------------
//  Local Functions
//-------------------------------------------------------------------------------------------------

//-------------------------------------------------------------------------------------------------
//  Static Functions
//-------------------------------------------------------------------------------------------------
/**
 * Allocates a fresh unused token from the token pool.
 */
static jsmntok_t *jsmn_alloc_token(jsmn_parser *parser,
        jsmntok_t *tokens, size_t num_tokens) {
    jsmntok_t *tok;
    if (parser->toknext >= num_tokens) {
        return NULL;
    }
    tok = &tokens[parser->toknext++];
    tok->start = tok->end = -1;
    tok->size = 0;
#ifdef JSMN_PARENT_LINKS
    tok->parent = -1;
#endif
    return tok;
}

/**
 * Fills token type and boundaries.
 */
static void jsmn_fill_token(jsmntok_t *token, jsmntype_t type,
                            int start, int end) {
    token->type = type;
    token->start = start;
    token->end = end;
    token->size = 0;
}

/**
 * Fills next available token with JSON primitive.
 */
static int jsmn_parse_primitive(jsmn_parser *parser, const char *js,
        size_t len, jsmntok_t *tokens, size_t num_tokens) {
    jsmntok_t *token;
    int start;

    start = parser->pos;

    for (; parser->pos < len && js[parser->pos] != '\0'; parser->pos++) {
        switch (js[parser->pos]) {
#ifndef JSMN_STRICT
            /* In strict mode primitive must be followed by "," or "}" or "]" */
            case ':':
#endif
            case '\t' : case '\r' : case '\n' : case ' ' :
            case ','  : case ']'  : case '}' :
                goto found;
        }
        if (js[parser->pos] < 32 || js[parser->pos] >= 127) {
            parser->pos = start;
            return JSMN_ERROR_INVAL;
        }
    }
#ifdef JSMN_STRICT
    /* In strict mode primitive must be followed by a comma/object/array */
    parser->pos = start;
    return JSMN_ERROR_PART;
#endif

found:
    if (tokens == NULL) {
        parser->pos--;
        return 0;
    }
    token = jsmn_alloc_token(parser, tokens, num_tokens);
    if (token == NULL) {
        parser->pos = start;
        return JSMN_ERROR_NOMEM;
    }
    jsmn_fill_token(token, JSMN_PRIMITIVE, start, parser->pos);
#ifdef JSMN_PARENT_LINKS
    token->parent = parser->toksuper;
#endif
    parser->pos--;
    return 0;
}

/**
 * Fills next token with JSON string.
 */
static int jsmn_parse_string(jsmn_parser *parser, const char *js,
        size_t len, jsmntok_t *tokens, size_t num_tokens) {
    jsmntok_t *token;

    int start = parser->pos;

    parser->pos++;

    /* Skip starting quote */
    for (; parser->pos < len && js[parser->pos] != '\0'; parser->pos++) {
        char c = js[parser->pos];

        /* Quote: end of string */
        if (c == '\"') {
            if (tokens == NULL) {
                return 0;
            }
            token = jsmn_alloc_token(parser, tokens, num_tokens);
            if (token == NULL) {
                parser->pos = start;
                return JSMN_ERROR_NOMEM;
            }
            jsmn_fill_token(token, JSMN_STRING, start+1, parser->pos);
#ifdef JSMN_PARENT_LINKS
            token->parent = parser->toksuper;
#endif
            return 0;
        }

        /* Backslash: Quoted symbol expected */
        if (c == '\\' && parser->pos + 1 < len) {
            int i;
            parser->pos++;
            switch (js[parser->pos]) {
                /* Allowed escaped symbols */
                case '\"': case '/' : case '\\' : case 'b' :
                case 'f' : case 'r' : case 'n'  : case 't' :
                    break;
                /* Allows escaped symbol \uXXXX */
                case 'u':
                    parser->pos++;
                    for(i = 0; i < 4 && parser->pos < len && js[parser->pos] != '\0'; i++) {
                        /* If it isn't a hex character we have an error */
                        if(!((js[parser->pos] >= 48 && js[parser->pos] <= 57) || /* 0-9 */
                                    (js[parser->pos] >= 65 && js[parser->pos] <= 70) || /* A-F */
                                    (js[parser->pos] >= 97 && js[parser->pos] <= 102))) { /* a-f */
                            parser->pos = start;
                            return JSMN_ERROR_INVAL;
                        }
                        parser->pos++;
                    }
                    parser->pos--;
                    break;
                /* Unexpected symbol */
                default:
                    parser->pos = start;
                    return JSMN_ERROR_INVAL;
            }
        }
    }
    parser->pos = start;
    return JSMN_ERROR_PART;
}

static int _jsmn_append_indent(char *buf, unsigned int bufMaxLen, int indentlevel, int indent) {
    int i;
    int writtenTotal = 0;
    char *indentString = NULL;
    int ret;

    if(NULL == buf)
    {
        return JSMN_ERROR_INVAL;
    }

    indentString = calloc(indent + 1, sizeof(char));
    if(NULL == indentString)
    {
        return JSMN_ERROR_NOMEM;
    }
    memset(indentString, ' ', indent);

    for(i = 0; i < indentlevel; i++)
    {
        ret = jsmn_snprintf(&buf, &bufMaxLen, &writtenTotal, "%s", indentString);

        if (ret != 0)
            JSMN_DEBUG("%s fail, indentlevel = %d, ret = %d\n", __func__, indentlevel, ret);
    }

    if(indentString)
    {
        free(indentString);
        indentString = NULL;
    }

    return writtenTotal;
}

//-------------------------------------------------------------------------------------------------
//  Global Functions
//-------------------------------------------------------------------------------------------------
/**
 * Parse JSON string and fill tokens.
 */
int jsmn_parse(jsmn_parser *parser, const char *js, size_t len,
        jsmntok_t *tokens, unsigned int num_tokens) {
    int r;
    int i;
    jsmntok_t *token;
    int count = parser->toknext;

    for (; parser->pos < len && js[parser->pos] != '\0'; parser->pos++) {
        char c;
        jsmntype_t type;

        c = js[parser->pos];
        switch (c) {
            case '{': case '[':
                count++;
                if (tokens == NULL) {
                    break;
                }
                token = jsmn_alloc_token(parser, tokens, num_tokens);
                if (token == NULL)
                    return JSMN_ERROR_NOMEM;
                if (parser->toksuper != -1) {
                    tokens[parser->toksuper].size++;
#ifdef JSMN_PARENT_LINKS
                    token->parent = parser->toksuper;
#endif
                }
                token->type = (c == '{' ? JSMN_OBJECT : JSMN_ARRAY);
                token->start = parser->pos;
                parser->toksuper = parser->toknext - 1;
                break;
            case '}': case ']':
                if (tokens == NULL)
                    break;
                type = (c == '}' ? JSMN_OBJECT : JSMN_ARRAY);
#ifdef JSMN_PARENT_LINKS
                if (parser->toknext < 1) {
                    return JSMN_ERROR_INVAL;
                }
                token = &tokens[parser->toknext - 1];
                for (;;) {
                    if (token->start != -1 && token->end == -1) {
                        if (token->type != type) {
                            return JSMN_ERROR_INVAL;
                        }
                        token->end = parser->pos + 1;
                        parser->toksuper = token->parent;
                        break;
                    }
                    if (token->parent == -1) {
                        if(token->type != type || parser->toksuper == -1) {
                            return JSMN_ERROR_INVAL;
                        }
                        break;
                    }
                    token = &tokens[token->parent];
                }
#else
                for (i = parser->toknext - 1; i >= 0; i--) {
                    token = &tokens[i];
                    if (token->start != -1 && token->end == -1) {
                        if (token->type != type) {
                            return JSMN_ERROR_INVAL;
                        }
                        parser->toksuper = -1;
                        token->end = parser->pos + 1;
                        break;
                    }
                }
                /* Error if unmatched closing bracket */
                if (i == -1) return JSMN_ERROR_INVAL;
                for (; i >= 0; i--) {
                    token = &tokens[i];
                    if (token->start != -1 && token->end == -1) {
                        parser->toksuper = i;
                        break;
                    }
                }
#endif
                break;
            case '\"':
                r = jsmn_parse_string(parser, js, len, tokens, num_tokens);
                if (r < 0) return r;
                count++;
                if (parser->toksuper != -1 && tokens != NULL)
                    tokens[parser->toksuper].size++;
                break;
            case '\t' : case '\r' : case '\n' : case ' ':
                break;
            case ':':
                parser->toksuper = parser->toknext - 1;
                break;
            case ',':
                if (tokens != NULL && parser->toksuper != -1 &&
                        tokens[parser->toksuper].type != JSMN_ARRAY &&
                        tokens[parser->toksuper].type != JSMN_OBJECT) {
#ifdef JSMN_PARENT_LINKS
                    parser->toksuper = tokens[parser->toksuper].parent;
#else
                    for (i = parser->toknext - 1; i >= 0; i--) {
                        if (tokens[i].type == JSMN_ARRAY || tokens[i].type == JSMN_OBJECT) {
                            if (tokens[i].start != -1 && tokens[i].end == -1) {
                                parser->toksuper = i;
                                break;
                            }
                        }
                    }
#endif
                }
                break;
#ifdef JSMN_STRICT
            /* In strict mode primitives are: numbers and booleans */
            case '-': case '0': case '1' : case '2': case '3' : case '4':
            case '5': case '6': case '7' : case '8': case '9':
            case 't': case 'f': case 'n' :
                /* And they must not be keys of the object */
                if (tokens != NULL && parser->toksuper != -1) {
                    jsmntok_t *t = &tokens[parser->toksuper];
                    if (t->type == JSMN_OBJECT ||
                            (t->type == JSMN_STRING && t->size != 0)) {
                        return JSMN_ERROR_INVAL;
                    }
                }
#else
            /* In non-strict mode every unquoted value is a primitive */
            default:
#endif
                r = jsmn_parse_primitive(parser, js, len, tokens, num_tokens);
                if (r < 0) return r;
                count++;
                if (parser->toksuper != -1 && tokens != NULL)
                    tokens[parser->toksuper].size++;
                break;

#ifdef JSMN_STRICT
            /* Unexpected char in strict mode */
            default:
                return JSMN_ERROR_INVAL;
#endif
        }
    }

    if (tokens != NULL) {
        for (i = parser->toknext - 1; i >= 0; i--) {
            /* Unmatched opened object or array */
            if (tokens[i].start != -1 && tokens[i].end == -1) {
                return JSMN_ERROR_PART;
            }
        }
    }

    return count;
}

/**
 * Creates a new parser based over a given  buffer with an array of tokens
 * available.
 */
void jsmn_init(jsmn_parser *parser) {
    parser->pos = 0;
    parser->toknext = 0;
    parser->toksuper = -1;
}

/*
 * Read JSON from stdin and print its content to stdout.
 * The output looks like YAML, but I'm not sure if it's really compatible.
 */
int jsmn_dump(const char *js, jsmntok_t *t, size_t count, int indent) {
    int i, j, k;
    if (count == 0) {
        return 0;
    }
    if (t->type == JSMN_PRIMITIVE) {
        printf("%.*s", t->end - t->start, js+t->start);
        return 1;
    } else if (t->type == JSMN_STRING) {
        printf("'%.*s'", t->end - t->start, js+t->start);
        return 1;
    } else if (t->type == JSMN_OBJECT) {
        printf("\n");
        j = 0;
        for (i = 0; i < t->size; i++) {
            for (k = 0; k < indent; k++) printf("  ");
            j += jsmn_dump(js, t+1+j, count-j, indent+1);
            printf(": ");
            j += jsmn_dump(js, t+1+j, count-j, indent+1);
            printf("\n");
        }
        return j+1;
    } else if (t->type == JSMN_ARRAY) {
        j = 0;
        printf("\n");
        for (i = 0; i < t->size; i++) {
            for (k = 0; k < indent-1; k++) printf("  ");
            printf("   - ");
            j += jsmn_dump(js, t+1+j, count-j, indent+1);
            printf("\n");
        }
        return j+1;
    }
    return 0;
}

int jsmn_eq(const char *json, jsmntok_t *tok, const char *s) {
    if (tok->type == JSMN_STRING && (int) strlen(s) == tok->end - tok->start &&
            strncmp(json + tok->start, s, tok->end - tok->start) == 0) {
        return 0;
    }
    return -1;
}

int jsmn_puts(char **pBuf, unsigned int *pBufMaxLen, int *pWrittenTotal, int written)
{
    if(NULL == pBuf)
    {
        JSMN_DEBUG("pBuf is NULL.\n");
        return JSMN_ERROR_INVAL;
    }
    if(NULL == (*pBuf))
    {
        JSMN_DEBUG("(*pBuf) is NULL.\n");
        return JSMN_ERROR_INVAL;
    }
    if(NULL == pBufMaxLen)
    {
        JSMN_DEBUG("pBufMaxLen is NULL.\n");
        return JSMN_ERROR_INVAL;
    }
    if(NULL == pWrittenTotal)
    {
        JSMN_DEBUG("pWrittenTotal is NULL.\n");
        return JSMN_ERROR_INVAL;
    }
    if(0 > written)
    {
        JSMN_DEBUG("written < 0.\n");
        return JSMN_ERROR_INVAL;
    }
    if((*pBufMaxLen) <= written)
    {
        JSMN_DEBUG("written(%d) >= *pBufMaxLen(%d). String can't be terminated.\n", written, *pBufMaxLen);
        return JSMN_ERROR_NOMEM;
    }

    (*pBuf) += written;
    (*pBufMaxLen) -= written;
    (*pWrittenTotal) += written;

    return 0;
}

int jsmn_append_string(char *buf, unsigned int bufMaxLen, const char *string) {
    if(NULL == buf)
    {
        return JSMN_ERROR_INVAL;
    }
    if(NULL == string)
    {
        return JSMN_ERROR_INVAL;
    }

    return snprintf(buf, bufMaxLen, "%s", string);
}

int jsmn_snprintf(char **pBuf, unsigned int *pBufMaxLen, int *pWrittenTotal, char const* fmt, ...)
{
    int ret;
    int written;
    char tmpString[JSON_MAX_SERIALIZE_LEN];
    va_list args;

    if(NULL == pBuf)
    {
        JSMN_DEBUG("pBuf is NULL.\n");
        return JSMN_ERROR_INVAL;
    }
    if(NULL == (*pBuf))
    {
        JSMN_DEBUG("(*pBuf) is NULL.\n");
        return JSMN_ERROR_INVAL;
    }
    if(NULL == pBufMaxLen)
    {
        JSMN_DEBUG("pBufMaxLen is NULL.\n");
        return JSMN_ERROR_INVAL;
    }
    if(NULL == pWrittenTotal)
    {
        JSMN_DEBUG("pWrittenTotal is NULL.\n");
        return JSMN_ERROR_INVAL;
    }

    memset(tmpString, 0, sizeof(tmpString));

    va_start(args, fmt);
    if(JSON_MAX_SERIALIZE_LEN <= vsnprintf(tmpString, JSON_MAX_SERIALIZE_LEN, fmt, args)) \
    {
        JSMN_DEBUG("vsnprintf(...) is failed.\n");
        va_end(args);
        return JSMN_ERROR_NOMEM;
    }
    va_end(args);

    written = jsmn_append_string(*pBuf, *pBufMaxLen, tmpString);
    ret = jsmn_puts(pBuf, pBufMaxLen, pWrittenTotal, written);
    if(0 > ret)
    {
        JSMN_DEBUG("jsmn_puts(...) is failed. ret = %d.\n", ret);
    }

    return ret;
}

int jsmn_append_indent(char **pBuf, unsigned int *pBufMaxLen, int *pWrittenTotal, int indentlevel, int indent)
{
    int ret;
    int written;

    if(NULL == pBuf)
    {
        JSMN_DEBUG("pBuf is NULL.\n");
        return JSMN_ERROR_INVAL;
    }
    if(NULL == (*pBuf))
    {
        JSMN_DEBUG("(*pBuf) is NULL.\n");
        return JSMN_ERROR_INVAL;
    }
    if(NULL == pBufMaxLen)
    {
        JSMN_DEBUG("pBufMaxLen is NULL.\n");
        return JSMN_ERROR_INVAL;
    }
    if(NULL == pWrittenTotal)
    {
        JSMN_DEBUG("pWrittenTotal is NULL.\n");
        return JSMN_ERROR_INVAL;
    }

    written = _jsmn_append_indent(*pBuf, *pBufMaxLen, indentlevel, indent);
    ret = jsmn_puts(pBuf, pBufMaxLen, pWrittenTotal, written);
    if(0 > ret)
    {
        JSMN_DEBUG("jsmn_puts(...) is failed. ret = %d.\n", ret);
    }

    return ret;
}
