///////////////////////////////////////////////////////////////////////////////////////////////////
/// @file   vector.c
/// @brief  vector implemented in C
/// @author MediaTek Inc.
///////////////////////////////////////////////////////////////////////////////////////////////////

//-------------------------------------------------------------------------------------------------
//  Include Files
//-------------------------------------------------------------------------------------------------
#include <stdio.h>
#include <stdlib.h>
#include <linux/string.h>

#include <MsTypes.h>

#include <common.h>
#include <command.h>
#include <malloc.h>
#include <MsDebug.h>

#include <bootlogo/vector.h>

#define OSAL_OS_MALLOC_V(size)                          malloc(size)
#define OSAL_OS_CALLOC_V(num, size)                     calloc(num, size)
#define OSAL_OS_FREE_V(ptr)                             free(ptr)

//-------------------------------------------------------------------------------------------------
//  Local Compiler Options
//-------------------------------------------------------------------------------------------------
//#define VECTOR_DEBUG

//-------------------------------------------------------------------------------------------------
//  Debug Defines
//-------------------------------------------------------------------------------------------------
#ifdef IS_NULL
#undef IS_NULL
#endif //#ifdef IS_NULL
#ifdef IS_NOT_NULL
#undef IS_NOT_NULL
#endif //#ifdef IS_NOT_NULL
#define IS_NULL(pointer)                                (NULL == (pointer))
#define IS_NOT_NULL(pointer)                            (!(IS_NULL(pointer)))

#ifndef ASSERT_FAIL_RETURN_FALSE
#define ASSERT_FAIL_RETURN_FALSE(_x_)                                                            \
    do  {                                                                                   \
        if ( ! ( _x_ ) )                                                                    \
        {                                                                                   \
            UBOOT_DEBUG("ASSERT FAIL RETURN FALSE: %s %s %d\n", __FILE__, __PRETTY_FUNCTION__, __LINE__);     \
            return FALSE;                                                                   \
        }                                                                                   \
    } while (0)
#endif //#ifndef ASSERT_FAIL_RETURN_FALSE

#ifndef ASSERT_FAIL_RETURN_
#define ASSERT_FAIL_RETURN(_x_, ret_value)                                                            \
    do  {                                                                                   \
        if ( ! ( _x_ ) )                                                                    \
        {                                                                                   \
            UBOOT_DEBUG("ASSERT FAIL RETURN FALSE: %s %s %d\n", __FILE__, __PRETTY_FUNCTION__, __LINE__);     \
            return (ret_value);                                                                   \
        }                                                                                   \
    } while (0)
#endif //#ifndef ASSERT_FAIL_RETURN_

#ifndef CHECK_PARAM_NULL_RETURN_VOID
#define CHECK_PARAM_NULL_RETURN_VOID(parameter) \
    do \
    { \
        if(IS_NULL(parameter)) \
        { \
            UBOOT_DEBUG("%s: %d: " #parameter " is NULL.\n", __func__, __LINE__); \
            return; \
        } \
    } while(0)
#endif //#ifndef CHECK_PARAM_NULL_RETURN_VOID

#ifndef CHECK_PARAM_NULL_RETURN
#define CHECK_PARAM_NULL_RETURN(parameter, ret_value) \
    do \
    { \
        if(IS_NULL(parameter)) \
        { \
            UBOOT_DEBUG("%s: %d: " #parameter " is NULL.\n", __func__, __LINE__); \
            return (ret_value); \
        } \
    } while(0)
#endif //#ifndef CHECK_PARAM_NULL_RETURN

#ifndef CHECK_PARAM_RETURN
#define CHECK_PARAM_RETURN(parameter, target, ret_value) \
    do \
    { \
        if((parameter) == (target)) \
        { \
            UBOOT_DEBUG("%s: %d: " #parameter " is NULL.\n", __func__, __LINE__); \
            return (ret_value); \
        } \
    } while(0)
#endif //#ifndef CHECK_PARAM_RETURN

//-------------------------------------------------------------------------------------------------
//  Local Defines
//-------------------------------------------------------------------------------------------------
#define VECTOR_IS_NULL(pointer)                         (NULL == (pointer))
#define VECTOR_IS_NOT_NULL(pointer)                     (!(VECTOR_IS_NULL(pointer)))

#define VECTOR_INIT_CAPACITY                            5

//-------------------------------------------------------------------------------------------------
//  Local Structures
//-------------------------------------------------------------------------------------------------

//-------------------------------------------------------------------------------------------------
//  Global Variables
//-------------------------------------------------------------------------------------------------

//-------------------------------------------------------------------------------------------------
//  Local Variables
//-------------------------------------------------------------------------------------------------

//-------------------------------------------------------------------------------------------------
//  Local Functions
//-------------------------------------------------------------------------------------------------

//-------------------------------------------------------------------------------------------------
//  Static Functions
//-------------------------------------------------------------------------------------------------
static MS_BOOL _VECTOR_Resize(vector_Vector_t *pstVector, MS_U16 u16Capacity)
{
    void **pItems;

    if(u16Capacity == pstVector->u16Capacity)
    {
        UBOOT_DEBUG("%s: The capacity is not changed.\n", __func__);
        return FALSE;
    }
    if(pstVector->u16Size > u16Capacity)
    {
        UBOOT_DEBUG("%s: The new capacity is less than the size of existed elements.\n", __func__);
        return FALSE;
    }

    UBOOT_INFO("%s: old(%d), new(%d)\n", __func__, pstVector->u16Capacity, u16Capacity);

    pItems = realloc(pstVector->pItems, sizeof(void *) * u16Capacity);
    ASSERT_FAIL_RETURN_FALSE(VECTOR_IS_NOT_NULL(pItems));

    if(u16Capacity > pstVector->u16Capacity)
    {
        UBOOT_INFO("%s: pItems = %p\n", __func__, pItems);
        UBOOT_INFO("%s: sizeof(void *) = %u\n", __func__, (unsigned)sizeof(void *));
        UBOOT_INFO("%s: &pItems[%u] = %p\n", __func__, pstVector->u16Size, &pItems[pstVector->u16Size]);
        UBOOT_INFO("%s: u16Capacity = %u\n", __func__, u16Capacity);
        UBOOT_INFO("%s: pstVector->u16Size = %u\n", __func__, pstVector->u16Size);

        memset(&pItems[pstVector->u16Size], 0, sizeof(void *) * (u16Capacity - pstVector->u16Size));
    }

    pstVector->pItems = pItems;
    pstVector->u16Capacity = u16Capacity;

    return TRUE;
}

static void _VECTOR_PrintItem(vector_Vector_t* pstVector, MS_U16 u16Idx, vector_ItemType_e eItemType)
{
    switch(eItemType)
    {
        case E_VECTOR_ITEM_TYPE_STRING:
            UBOOT_INFO("%s ", (char*) vector_At(pstVector, u16Idx));
            break;

        default:
            UBOOT_INFO("eItemType(%d) is not supported.\n", eItemType);
            break;
    }

    return;
}

void _VECTOR_FreeItem(vector_Vector_t* pstVector, MS_U16 u16Idx)
{
    void* pItem;

    pItem = vector_At(pstVector, u16Idx);
    if(pItem)
    {
        OSAL_OS_FREE_V(pItem);
        pstVector->pItems[u16Idx] = NULL;
    }
    else
    {
        UBOOT_DEBUG("%s: Item(%d) is NULL.\n", __func__, u16Idx);
    }

    return;
}

//-------------------------------------------------------------------------------------------------
//  Global Functions
//-------------------------------------------------------------------------------------------------
vector_Vector_t *vector_Init(void)
{
    vector_Vector_t *pstVector;

    pstVector = OSAL_OS_MALLOC_V(sizeof(vector_Vector_t));
    ASSERT_FAIL_RETURN(VECTOR_IS_NOT_NULL(pstVector), NULL);
    pstVector->u16Capacity = VECTOR_INIT_CAPACITY;
    pstVector->u16Size = 0;
    pstVector->pItems = OSAL_OS_CALLOC_V(pstVector->u16Capacity, sizeof(void *));
    if(VECTOR_IS_NULL(pstVector->pItems))
    {
        UBOOT_DEBUG("%s %s %d\n: Fail to calloc(...) of pstVector->u16Capacity.\n", __FILE__, __PRETTY_FUNCTION__, __LINE__);     \
        OSAL_OS_FREE_V(pstVector);

        return NULL;
    }

    return pstVector;
}

void vector_DeInit(vector_Vector_t *pstVector)
{
    int i;

    CHECK_PARAM_NULL_RETURN_VOID(pstVector);

    for(i = 0; i < vector_Size(pstVector); i++)
    {
        _VECTOR_FreeItem(pstVector, i);
    }
    OSAL_OS_FREE_V(pstVector->pItems);
    OSAL_OS_FREE_V(pstVector);

    return;
}

MS_U16 vector_Size(vector_Vector_t *pstVector)
{
    CHECK_PARAM_NULL_RETURN(pstVector, 0);

    return pstVector->u16Size;
}

MS_BOOL vector_PushBack(vector_Vector_t *pstVector, void *pItem, MS_U32 u32MemorySize)
{
    void *pNewItem = NULL;

    CHECK_PARAM_NULL_RETURN(pstVector, FALSE);
    CHECK_PARAM_NULL_RETURN(pItem, FALSE);
    CHECK_PARAM_RETURN(u32MemorySize, 0, FALSE);

    if (pstVector->u16Capacity <= pstVector->u16Size)
    {
        if(!_VECTOR_Resize(pstVector, pstVector->u16Capacity * 2))
        {
            UBOOT_DEBUG("_VECTOR_Resize is failed.\n");

            return FALSE;
        }
    }

    pNewItem = OSAL_OS_MALLOC_V(u32MemorySize);
    ASSERT_FAIL_RETURN_FALSE(VECTOR_IS_NOT_NULL(pNewItem));
    pstVector->pItems[pstVector->u16Size++] = memcpy(pNewItem, pItem, u32MemorySize);

    return TRUE;
}

MS_BOOL vector_Assign(vector_Vector_t *pstVector, MS_U16 u16Idx, void *pItem, MS_U32 u32MemorySize)
{
    void *pNewItem = NULL;

    CHECK_PARAM_NULL_RETURN(pstVector, FALSE);

    if(u16Idx >= pstVector->u16Size)
    {
        UBOOT_DEBUG("%s: %d: Idx(%u) is out of range.\n", __func__, __LINE__, u16Idx);
        return FALSE;
    }
    CHECK_PARAM_NULL_RETURN(pItem, FALSE);
    CHECK_PARAM_RETURN(u32MemorySize, 0, FALSE);

    pNewItem = OSAL_OS_MALLOC_V(u32MemorySize);
    ASSERT_FAIL_RETURN_FALSE(VECTOR_IS_NOT_NULL(pNewItem));
    _VECTOR_FreeItem(pstVector, u16Idx);
    pstVector->pItems[u16Idx] = memcpy(pNewItem, pItem, u32MemorySize);

    return TRUE;
}

void* vector_At(vector_Vector_t* pstVector, MS_U16 u16Idx)
{
    CHECK_PARAM_NULL_RETURN(pstVector, NULL);

    if(u16Idx >= pstVector->u16Size)
    {
        UBOOT_DEBUG("%s: %d: Idx(%u) is out of range.\n", __func__, __LINE__, u16Idx);
        return NULL;
    }

    return pstVector->pItems[u16Idx];
}

MS_BOOL vector_Erase(vector_Vector_t *pstVector, MS_U16 u16Idx)
{
    int i;

    CHECK_PARAM_NULL_RETURN(pstVector, FALSE);
    if(u16Idx >= pstVector->u16Size)
    {
        UBOOT_DEBUG("%s: %d: Idx(%u) is out of range.\n", __func__, __LINE__, u16Idx);
        return FALSE;
    }

    _VECTOR_FreeItem(pstVector, u16Idx);

    for(i = u16Idx; i < pstVector->u16Size - 1; i++)
    {
        pstVector->pItems[i] = pstVector->pItems[i + 1];
        pstVector->pItems[i + 1] = NULL;
    }

    pstVector->u16Size--;

    if((pstVector->u16Size > 0) && (pstVector->u16Size == pstVector->u16Capacity / 4))
    {
        _VECTOR_Resize(pstVector, pstVector->u16Capacity / 2);
    }

    return TRUE;
}

void vector_Print(vector_Vector_t* pstVector, vector_ItemType_e eItemType)
{
    int i;

    CHECK_PARAM_NULL_RETURN_VOID(pstVector);

    for(i = 0; i < vector_Size(pstVector); i++)
    {
        _VECTOR_PrintItem(pstVector, i, eItemType);
    }
    UBOOT_INFO("\n");

    return;
}

#ifdef VECTOR_DEBUG
int main(void)
{
    vector_Vector_t *pstVector = vector_Init();

    vector_PushBack(pstVector, "Zero", (strlen("Zero") + 1) * sizeof(char));
    vector_PushBack(pstVector, "One", (strlen("One") + 1) * sizeof(char));
    vector_PushBack(pstVector, "Two", (strlen("Two") + 1) * sizeof(char));
    vector_PushBack(pstVector, "Three", (strlen("Three") + 1) * sizeof(char));
    vector_PushBack(pstVector, "Four", (strlen("Four") + 1) * sizeof(char));
    vector_PushBack(pstVector, "Five", (strlen("Five") + 1) * sizeof(char));
    vector_PushBack(pstVector, "Six", (strlen("Six") + 1) * sizeof(char));
    vector_Print(pstVector, E_VECTOR_ITEM_TYPE_STRING);

    vector_Erase(pstVector, 1);
    vector_Erase(pstVector, 1);
    vector_Erase(pstVector, 1);
    vector_Erase(pstVector, 1);
    vector_Erase(pstVector, 1);
    vector_Print(pstVector, E_VECTOR_ITEM_TYPE_STRING);

    vector_Assign(pstVector, 0, "Hello", (strlen("Hello") + 1) * sizeof(char));
    vector_PushBack(pstVector, "World", (strlen("World") + 1) * sizeof(char));
    vector_Print(pstVector, E_VECTOR_ITEM_TYPE_STRING);

    vector_DeInit(pstVector);
}
#endif //#ifdef VECTOR_DEBUG
